﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Transformations {
  [Item("ShiftDataToRangeTransformation", "Represents a linear Transformation using Parameters defining a target range")]
  public class ShiftDataToRangeTransformation : LinearTransformation {
    protected const string RangeParameterName = "Range";

    #region Parameters
    public IValueParameter<DoubleRange> RangeParameter {
      get { return (IValueParameter<DoubleRange>)Parameters[RangeParameterName]; }
    }
    #endregion

    #region properties
    public DoubleRange Range {
      get { return RangeParameter.Value; }
    }
    #endregion

    [StorableConstructor]
    protected ShiftDataToRangeTransformation(bool deserializing) : base(deserializing) { }
    protected ShiftDataToRangeTransformation(Transformation<double> original, Cloner cloner)
      : base(original, cloner) {
    }
    protected ShiftDataToRangeTransformation()
      : base() {
    }
    public ShiftDataToRangeTransformation(IEnumerable<string> allowedColumns)
      : base(allowedColumns) {
      MultiplicandParameter.Hidden = true;
      SummandParameter.Hidden = true;
      Parameters.Add(new ValueParameter<DoubleRange>(RangeParameterName, "Range for the target window of the linear transformation", new DoubleRange(0.0, 1.0)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ShiftDataToRangeTransformation(this, cloner);
    }

    public override IEnumerable<double> Apply(IEnumerable<double> data) {
      ConfigureParameters(data);
      return base.Apply(data);
    }

    public override bool Check(IEnumerable<double> data, out string errorMsg) {
      ConfigureParameters(data);
      return base.Check(data, out errorMsg);
    }

    protected void ConfigureParameters(IEnumerable<double> data) {
      double originalRangeStart = data.Min();
      double originalRangeEnd = data.Max();

      double originalRangeWidth = originalRangeEnd - originalRangeStart;
      double targetRangeWidth = Range.End - Range.Start;

      Multiplicand = targetRangeWidth / originalRangeWidth;
      Summand = Range.Start - originalRangeStart * Multiplicand;
    }
  }
}
